@@ -128,13 +128,14 @@ char* buffer_string_prepare_copy(buffer *b, size_t size) {
 char* buffer_string_prepare_append(buffer *b, size_t size) {
 	force_assert(NULL !=  b);
 
+	if (b->used && size < b->size - b->used)
+		return b->ptr + b->used - 1;
+
 	if (buffer_string_is_empty(b)) {
 		return buffer_string_prepare_copy(b, size);
 	} else {
-		size_t req_size = b->used + size;
-
 		/* not empty, b->used already includes a terminating 0 */
-		force_assert(req_size >= b->used);
+		size_t req_size = b->used + size;
 
 		/* check for overflow: unsigned overflow is defined to wrap around */
 		force_assert(req_size >= b->used);
@@ -147,9 +148,11 @@ char* buffer_string_prepare_append(buffer *b, size_t size) {
 
 void buffer_string_set_length(buffer *b, size_t len) {
 	force_assert(NULL != b);
-	force_assert(len + 1 > len);
 
-	buffer_realloc(b, len + 1);
+	if (len >= b->size) {
+		force_assert(len + 1 > len);
+		buffer_realloc(b, len + 1);
+	}
 
 	b->used = len + 1;
 	b->ptr[len] = '\0';
@@ -183,9 +186,9 @@ void buffer_copy_string_len(buffer *b, const char *s, size_t s_len) {
 
 	buffer_string_prepare_copy(b, s_len);
 
-	if (0 != s_len) memcpy(b->ptr, s, s_len);
-
-	buffer_commit(b, s_len);
+	if (0 != s_len) memcpy(b->ptr, s, s_len); /*(s might be NULL)*/
+	b->ptr[s_len] = '\0';
+	b->used = s_len + 1;
 }
 
 void buffer_copy_buffer(buffer *b, const buffer *src) {
@@ -220,11 +223,11 @@ void buffer_append_string_len(buffer *b, const char *s, size_t s_len) {
 
 	target_buf = buffer_string_prepare_append(b, s_len);
 
-	if (0 == s_len) return; /* nothing to append */
+	if (0 == s_len) return; /* nothing to append *//*(s might be NULL)*/
 
 	memcpy(target_buf, s, s_len);
-
-	buffer_commit(b, s_len);
+	target_buf[s_len] = '\0';
+	b->used += s_len;
 }
 
 void buffer_append_string_buffer(buffer *b, const buffer *src) {
@@ -377,38 +380,30 @@ char hex2int(unsigned char hex) {
 
 /**
  * check if two buffer contain the same data
- *
- * HISTORY: this function was pretty much optimized, but didn't handled
- * alignment properly.
  */
 
 int buffer_is_equal(const buffer *a, const buffer *b) {
 	force_assert(NULL != a && NULL != b);
 
-	if (a->used != b->used) return 0;
-	if (a->used == 0) return 1;
-
-	return (0 == memcmp(a->ptr, b->ptr, a->used));
+	/* 1 = equal; 0 = not equal */
+	return (a->used == b->used && 0 == memcmp(a->ptr, b->ptr, a->used));
 }
 
 int buffer_is_equal_string(const buffer *a, const char *s, size_t b_len) {
 	force_assert(NULL != a && NULL != s);
 	force_assert(b_len + 1 > b_len);
 
-	if (a->used != b_len + 1) return 0;
-	if (0 != memcmp(a->ptr, s, b_len)) return 0;
-	if ('\0' != a->ptr[a->used-1]) return 0;
-
-	return 1;
+	/* 1 = equal; 0 = not equal */
+	return (a->used == b_len + 1 && 0 == memcmp(a->ptr, s, b_len));
 }
 
 /* buffer_is_equal_caseless_string(b, CONST_STR_LEN("value")) */
 int buffer_is_equal_caseless_string(const buffer *a, const char *s, size_t b_len) {
-	force_assert(NULL != a);
-	if (a->used != b_len + 1) return 0;
-	force_assert('\0' == a->ptr[a->used - 1]);
+	force_assert(NULL != a && NULL != s);
+	force_assert(b_len + 1 > b_len);
 
-	return (0 == strcasecmp(a->ptr, s));
+	/* 1 = equal; 0 = not equal */
+	return (a->used == b_len + 1 && 0 == strncasecmp(a->ptr, s, b_len));
 }
 
 int buffer_caseless_compare(const char *a, size_t a_len, const char *b, size_t b_len) {
