@@ -10,33 +10,32 @@
 #include "configparser.h"
 #include "configfile.h"
 #include "stat_cache.h"
+#include "sys-crypto.h"
 
 #include <sys/stat.h>
+#include <sys/wait.h>
 
 #include <stdlib.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
 #include <string.h>
-#include <stdio.h>
 #include <ctype.h>
 #include <limits.h>
 #include <glob.h>
 
 
 #if defined(HAVE_MYSQL) || (defined(HAVE_LDAP_H) && defined(HAVE_LBER_H) && defined(HAVE_LIBLDAP) && defined(HAVE_LIBLBER))
-static void config_warn_authn_module (server *srv, const char *module) {
-	size_t len = strlen(module);
+static void config_warn_authn_module (server *srv, const char *module, size_t len) {
 	for (size_t i = 0; i < srv->config_context->used; ++i) {
 		const data_config *config = (data_config const*)srv->config_context->data[i];
 		const data_unset *du = array_get_element(config->value, "auth.backend");
 		if (NULL != du && du->type == TYPE_STRING) {
 			data_string *ds = (data_string *)du;
 			if (buffer_is_equal_string(ds->value, module, len)) {
-				ds = data_string_init();
-				buffer_copy_string_len(ds->value, CONST_STR_LEN("mod_authn_"));
-				buffer_append_string(ds->value, module);
-				array_insert_unique(srv->srvconf.modules, (data_unset *)ds);
+				buffer_copy_string_len(srv->tmp_buf, CONST_STR_LEN("mod_authn_"));
+				buffer_append_string_len(srv->tmp_buf, module, len);
+				array_insert_value(srv->srvconf.modules, CONST_BUF_LEN(srv->tmp_buf));
 				log_error_write(srv, __FILE__, __LINE__, "SSSsSSS", "Warning: please add \"mod_authn_", module, "\" to server.modules list in lighttpd.conf.  A future release of lighttpd 1.4.x will not automatically load mod_authn_", module, "and lighttpd will fail to start up since your lighttpd.conf uses auth.backend = \"", module, "\".");
 				return;
 			}
@@ -45,7 +44,7 @@ static void config_warn_authn_module (server *srv, const char *module) {
 }
 #endif
 
-#if defined HAVE_LIBSSL && defined HAVE_OPENSSL_SSL_H
+#ifdef USE_OPENSSL_CRYPTO
 static void config_warn_openssl_module (server *srv) {
 	for (size_t i = 0; i < srv->config_context->used; ++i) {
 		const data_config *config = (data_config const*)srv->config_context->data[i];
@@ -53,9 +52,7 @@ static void config_warn_openssl_module (server *srv) {
 			data_unset *du = config->value->data[j];
 			if (0 == strncmp(du->key->ptr, "ssl.", sizeof("ssl.")-1)) {
 				/* mod_openssl should be loaded after mod_extforward */
-				data_string *ds = data_string_init();
-				buffer_copy_string_len(ds->value, CONST_STR_LEN("mod_openssl"));
-				array_insert_unique(srv->srvconf.modules, (data_unset *)ds);
+				array_insert_value(srv->srvconf.modules, CONST_STR_LEN("mod_openssl"));
 				log_error_write(srv, __FILE__, __LINE__, "S", "Warning: please add \"mod_openssl\" to server.modules list in lighttpd.conf.  A future release of lighttpd 1.4.x *will not* automatically load mod_openssl and lighttpd *will not* use SSL/TLS where your lighttpd.conf contains ssl.* directives");
 				return;
 			}
@@ -506,14 +503,14 @@ static int config_insert(server *srv) {
 			if (HANDLER_ERROR == ret) break;
 		}
 
-#if !(defined HAVE_LIBSSL && defined HAVE_OPENSSL_SSL_H)
+	      #ifndef USE_OPENSSL_CRYPTO
 		if (s->ssl_enabled) {
 			log_error_write(srv, __FILE__, __LINE__, "s",
-					"ssl support is missing, recompile with --with-openssl");
+					"ssl support is missing, recompile with e.g. --with-openssl");
 			ret = HANDLER_ERROR;
 			break;
 		}
-#endif
+	      #endif
 	}
 	array_free(http_parseopts);
 
@@ -607,14 +604,11 @@ static int config_insert(server *srv) {
 		if (prepend_mod_indexfile) {
 			/* mod_indexfile has to be loaded before mod_fastcgi and friends */
 			array *modules = array_init();
-
-			ds = data_string_init();
-			buffer_copy_string_len(ds->value, CONST_STR_LEN("mod_indexfile"));
-			array_insert_unique(modules, (data_unset *)ds);
+			array_insert_value(modules, CONST_STR_LEN("mod_indexfile"));
 
 			for (i = 0; i < srv->srvconf.modules->used; i++) {
-				data_unset *du = srv->srvconf.modules->data[i];
-				array_insert_unique(modules, du->copy(du));
+				ds = (data_string *)srv->srvconf.modules->data[i];
+				array_insert_value(modules, CONST_BUF_LEN(ds->value));
 			}
 
 			array_free(srv->srvconf.modules);
@@ -623,19 +617,15 @@ static int config_insert(server *srv) {
 
 		/* append default modules */
 		if (append_mod_dirlisting) {
-			ds = data_string_init();
-			buffer_copy_string_len(ds->value, CONST_STR_LEN("mod_dirlisting"));
-			array_insert_unique(srv->srvconf.modules, (data_unset *)ds);
+			array_insert_value(srv->srvconf.modules, CONST_STR_LEN("mod_dirlisting"));
 		}
 
 		if (append_mod_staticfile) {
-			ds = data_string_init();
-			buffer_copy_string_len(ds->value, CONST_STR_LEN("mod_staticfile"));
-			array_insert_unique(srv->srvconf.modules, (data_unset *)ds);
+			array_insert_value(srv->srvconf.modules, CONST_STR_LEN("mod_staticfile"));
 		}
 
 		if (append_mod_openssl) {
-		      #if defined HAVE_LIBSSL && defined HAVE_OPENSSL_SSL_H
+		      #ifdef USE_OPENSSL_CRYPTO
 			config_warn_openssl_module(srv);
 		      #endif
 		}
@@ -645,18 +635,16 @@ static int config_insert(server *srv) {
 		 * existing lighttpd 1.4.x configs */
 		if (contains_mod_auth) {
 			if (append_mod_authn_file) {
-				ds = data_string_init();
-				buffer_copy_string_len(ds->value, CONST_STR_LEN("mod_authn_file"));
-				array_insert_unique(srv->srvconf.modules, (data_unset *)ds);
+				array_insert_value(srv->srvconf.modules, CONST_STR_LEN("mod_authn_file"));
 			}
 			if (append_mod_authn_ldap) {
 			      #if defined(HAVE_LDAP_H) && defined(HAVE_LBER_H) && defined(HAVE_LIBLDAP) && defined(HAVE_LIBLBER)
-				config_warn_authn_module(srv, "ldap");
+				config_warn_authn_module(srv, CONST_STR_LEN("ldap"));
 			      #endif
 			}
 			if (append_mod_authn_mysql) {
 			      #if defined(HAVE_MYSQL)
-				config_warn_authn_module(srv, "mysql");
+				config_warn_authn_module(srv, CONST_STR_LEN("mysql"));
 			      #endif
 			}
 		}
@@ -1405,12 +1393,9 @@ static char* getCWD(void) {
 }
 
 int config_parse_cmd(server *srv, config_t *context, const char *cmd) {
-	tokenizer_t t;
 	int ret = 0;
-	FILE *fp;
-	buffer *source;
-	buffer *out = srv->tmp_buf;
 	char *oldpwd;
+	int fds[2];
 
 	if (NULL == (oldpwd = getCWD())) {
 		log_error_write(srv, __FILE__, __LINE__, "s",
@@ -1427,44 +1412,74 @@ int config_parse_cmd(server *srv, config_t *context, const char *cmd) {
 		}
 	}
 
-	source = buffer_init_string(cmd);
-
-	fp = popen(cmd, "r");
-	if (NULL == fp) {
-		log_error_write(srv, __FILE__, __LINE__, "SSss",
-				"popen \"", cmd, "\"failed:", strerror(errno));
+	if (pipe(fds)) {
+		log_error_write(srv, __FILE__, __LINE__, "ss",
+				"pipe failed: ", strerror(errno));
 		ret = -1;
 	}
 	else {
-		size_t rd;
-		buffer_string_set_length(out, 0);
-		do {
-			rd = fread(buffer_string_prepare_append(out, 1023), 1, 1023, fp);
-			buffer_commit(out, rd);
-		} while (0 != rd && !ferror(fp));
-		if (0 != rd || !feof(fp)) {
+		char *shell = getenv("SHELL");
+		char *args[4];
+		pid_t pid;
+		*(const char **)&args[0] = shell ? shell : "/bin/sh";
+		*(const char **)&args[1] = "-c";
+		*(const char **)&args[2] = cmd;
+		args[3] = NULL;
+
+		fdevent_setfd_cloexec(fds[0]);
+		pid = fdevent_fork_execve(args[0], args, NULL, -1, fds[1], -1, -1);
+		if (-1 == pid) {
 			log_error_write(srv, __FILE__, __LINE__, "SSss",
-					"fread \"", cmd, "\"failed:", strerror(errno));
+					"fork/exec(", cmd, "):", strerror(errno));
 			ret = -1;
 		}
-		if (0 != pclose(fp)) {
-			log_error_write(srv, __FILE__, __LINE__, "SSss",
-					"pclose \"", cmd, "\"failed:", strerror(errno));
-			ret = -1;
+		else {
+			ssize_t rd;
+			pid_t wpid;
+			int wstatus;
+			buffer *out = srv->tmp_buf;
+			buffer_string_set_length(out, 0);
+			close(fds[1]);
+			fds[1] = -1;
+			do {
+				rd = read(fds[0], buffer_string_prepare_append(out, 1023), 1023);
+				if (rd >= 0) buffer_commit(out, (size_t)rd);
+			} while (rd > 0 || (-1 == rd && errno == EINTR));
+			if (0 != rd) {
+				log_error_write(srv, __FILE__, __LINE__, "SSss",
+						"read \"", cmd, "\" failed:", strerror(errno));
+				ret = -1;
+			}
+			close(fds[0]);
+			fds[0] = -1;
+			while (-1 == (wpid = waitpid(pid, &wstatus, 0)) && errno == EINTR) ;
+			if (wpid != pid) {
+				log_error_write(srv, __FILE__, __LINE__, "SSss",
+						"waitpid \"", cmd, "\" failed:", strerror(errno));
+				ret = -1;
+			}
+			if (0 != wstatus) {
+				log_error_write(srv, __FILE__, __LINE__, "SSsd",
+						"commaned \"", cmd, "\" exited non-zero:", WEXITSTATUS(wstatus));
+				ret = -1;
+			}
+
+			if (-1 != ret) {
+				buffer *source = buffer_init_string(cmd);
+				tokenizer_t t;
+				tokenizer_init(&t, source, CONST_BUF_LEN(out));
+				ret = config_parse(srv, context, &t);
+				buffer_free(source);
+			}
 		}
+		if (-1 != fds[0]) close(fds[0]);
+		if (-1 != fds[1]) close(fds[1]);
 	}
 
-	if (-1 != ret) {
-		tokenizer_init(&t, source, CONST_BUF_LEN(out));
-		ret = config_parse(srv, context, &t);
-	}
-
-	buffer_free(source);
 	if (0 != chdir(oldpwd)) {
 		log_error_write(srv, __FILE__, __LINE__, "sss",
 			"cannot change directory to", oldpwd, strerror(errno));
-		free(oldpwd);
-		return -1;
+		ret = -1;
 	}
 	free(oldpwd);
 	return ret;
@@ -1485,8 +1500,7 @@ static void context_free(config_t *context) {
 int config_read(server *srv, const char *fn) {
 	config_t context;
 	data_config *dc;
-	data_integer *dpid;
-	data_string *dcwd;
+	buffer *dcwd;
 	int ret;
 	char *pos;
 	buffer *filename;
@@ -1512,19 +1526,13 @@ int config_read(server *srv, const char *fn) {
 	context.current = dc;
 
 	/* default context */
-	dpid = data_integer_init();
-	dpid->value = getpid();
-	buffer_copy_string_len(dpid->key, CONST_STR_LEN("var.PID"));
-	array_insert_unique(dc->value, (data_unset *)dpid);
+	*array_get_int_ptr(dc->value, CONST_STR_LEN("var.PID")) = getpid();
 
-	dcwd = data_string_init();
-	buffer_string_prepare_copy(dcwd->value, 1023);
-	if (NULL != getcwd(dcwd->value->ptr, dcwd->value->size - 1)) {
-		buffer_commit(dcwd->value, strlen(dcwd->value->ptr));
-		buffer_copy_string_len(dcwd->key, CONST_STR_LEN("var.CWD"));
-		array_insert_unique(dc->value, (data_unset *)dcwd);
-	} else {
-		dcwd->free((data_unset*) dcwd);
+	dcwd = srv->tmp_buf;
+	buffer_string_prepare_copy(dcwd, 4095);
+	if (NULL != getcwd(dcwd->ptr, 4095)) {
+		buffer_commit(dcwd, strlen(dcwd->ptr));
+		array_set_key_value(dc->value, CONST_STR_LEN("var.CWD"), CONST_BUF_LEN(dcwd));
 	}
 
 	filename = buffer_init_string(fn);
@@ -1569,11 +1577,9 @@ int config_set_defaults(server *srv) {
 	}
 
 	if (!srv->srvconf.upload_tempdirs->used) {
-		data_string *ds = data_string_init();
 		const char *tmpdir = getenv("TMPDIR");
 		if (NULL == tmpdir) tmpdir = "/var/tmp";
-		buffer_copy_string(ds->value, tmpdir);
-		array_insert_unique(srv->srvconf.upload_tempdirs, (data_unset *)ds);
+		array_insert_value(srv->srvconf.upload_tempdirs, tmpdir, strlen(tmpdir));
 	}
 
 	if (srv->srvconf.upload_tempdirs->used) {
