@@ -56,7 +56,7 @@ static data_unset *configparser_get_variable(config_t *ctx, const buffer *key) {
     array_print(dc->value, 0);
 #endif
     if (NULL != (du = array_get_element_klen(dc->value, CONST_BUF_LEN(key)))) {
-      du = du->copy(du);
+      du = du->fn->copy(du);
       buffer_reset(du->key);
       return du;
     }
@@ -78,11 +78,11 @@ data_unset *configparser_merge_data(data_unset *op1, const data_unset *op2) {
       data_string *ds = data_string_init();
       buffer_append_int(ds->value, ((data_integer*)op1)->value);
       buffer_append_string_buffer(ds->value, ((data_string*)op2)->value);
-      op1->free(op1);
+      op1->fn->free(op1);
       return (data_unset *)ds;
     } else {
       fprintf(stderr, "data type mismatch, cannot merge\n");
-      op1->free(op1);
+      op1->fn->free(op1);
       return NULL;
     }
   }
@@ -104,10 +104,10 @@ data_unset *configparser_merge_data(data_unset *op1, const data_unset *op2) {
         du = (data_unset *)src->data[i];
         if (du) {
           if (du->is_index_key || buffer_is_empty(du->key) || !array_get_element_klen(dst, CONST_BUF_LEN(du->key))) {
-            array_insert_unique(dst, du->copy(du));
+            array_insert_unique(dst, du->fn->copy(du));
           } else {
             fprintf(stderr, "Duplicate array-key '%s'\n", du->key->ptr);
-            op1->free(op1);
+            op1->fn->free(op1);
             return NULL;
           }
         }
@@ -575,17 +575,17 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
       break;
     case 36:
 #line 177 "../../src/configparser.y"
-{ if ((yypminor->yy91)) (yypminor->yy91)->free((yypminor->yy91)); }
+{ if ((yypminor->yy91)) (yypminor->yy91)->fn->free((yypminor->yy91)); }
 #line 579 "./configparser.c"
       break;
     case 37:
 #line 178 "../../src/configparser.y"
-{ if ((yypminor->yy91)) (yypminor->yy91)->free((yypminor->yy91)); }
+{ if ((yypminor->yy91)) (yypminor->yy91)->fn->free((yypminor->yy91)); }
 #line 584 "./configparser.c"
       break;
     case 38:
 #line 179 "../../src/configparser.y"
-{ if ((yypminor->yy91)) (yypminor->yy91)->free((yypminor->yy91)); }
+{ if ((yypminor->yy91)) (yypminor->yy91)->fn->free((yypminor->yy91)); }
 #line 589 "./configparser.c"
       break;
     case 41:
@@ -919,7 +919,7 @@ static void yy_reduce(
   }
   buffer_free(yymsp[-2].minor.yy29);
   yymsp[-2].minor.yy29 = NULL;
-  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
+  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
 #line 925 "./configparser.c"
@@ -942,7 +942,7 @@ static void yy_reduce(
   }
   buffer_free(yymsp[-2].minor.yy29);
   yymsp[-2].minor.yy29 = NULL;
-  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
+  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
 #line 948 "./configparser.c"
@@ -977,7 +977,7 @@ static void yy_reduce(
   }
   buffer_free(yymsp[-2].minor.yy29);
   yymsp[-2].minor.yy29 = NULL;
-  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
+  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
 #line 983 "./configparser.c"
@@ -1009,9 +1009,9 @@ static void yy_reduce(
       ctx->ok = 0;
     }
   }
-  if (yymsp[-2].minor.yy91) yymsp[-2].minor.yy91->free(yymsp[-2].minor.yy91);
+  if (yymsp[-2].minor.yy91) yymsp[-2].minor.yy91->fn->free(yymsp[-2].minor.yy91);
   yymsp[-2].minor.yy91 = NULL;
-  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
+  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
 #line 1017 "./configparser.c"
@@ -1130,7 +1130,7 @@ static void yy_reduce(
   }
   array_free(yymsp[-2].minor.yy42);
   yymsp[-2].minor.yy42 = NULL;
-  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
+  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
 #line 1136 "./configparser.c"
@@ -1154,7 +1154,7 @@ static void yy_reduce(
     array_insert_unique(yygotominor.yy42, yymsp[0].minor.yy91);
     yymsp[0].minor.yy91 = NULL;
   }
-  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
+  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
 #line 1160 "./configparser.c"
@@ -1177,7 +1177,7 @@ static void yy_reduce(
     yygotominor.yy91 = yymsp[0].minor.yy91;
     yymsp[0].minor.yy91 = NULL;
   }
-  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
+  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
   buffer_free(yymsp[-2].minor.yy29);
   yymsp[-2].minor.yy29 = NULL;
@@ -1291,7 +1291,7 @@ static void yy_reduce(
     } else {
       fprintf(stderr, "unreachable else condition\n");
       ctx->ok = 0;
-      yymsp[0].minor.yy18->free((data_unset *)yymsp[0].minor.yy18);
+      yymsp[0].minor.yy18->fn->free((data_unset *)yymsp[0].minor.yy18);
       yymsp[0].minor.yy18 = dc;
     }
 
@@ -1386,7 +1386,7 @@ static void yy_reduce(
 
     b = buffer_init();
     buffer_copy_buffer(b, ctx->current->key);
-    buffer_append_string(b, "/");
+    buffer_append_string_len(b, CONST_STR_LEN("/"));
     buffer_append_string_buffer(b, yymsp[-5].minor.yy0);
     buffer_append_string_buffer(b, yymsp[-3].minor.yy29);
     buffer_append_string_buffer(b, op);
@@ -1535,7 +1535,7 @@ static void yy_reduce(
       if (ctx->ok) {
         configparser_push(ctx, dc, 1);
       } else {
-        dc->free((data_unset*) dc);
+        dc->fn->free((data_unset*) dc);
       }
     }
   }
@@ -1546,7 +1546,7 @@ static void yy_reduce(
   yymsp[-5].minor.yy0 = NULL;
   buffer_free(yymsp[-3].minor.yy29);
   yymsp[-3].minor.yy29 = NULL;
-  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
+  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
 #line 1552 "./configparser.c"
@@ -1616,7 +1616,7 @@ static void yy_reduce(
       ctx->ok = 0;
     }
   }
-  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
+  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
 #line 1622 "./configparser.c"
