@@ -35,7 +35,7 @@ static void data_integer_reset(data_unset *d) {
 static int data_integer_insert_dup(data_unset *dst, data_unset *src) {
 	UNUSED(dst);
 
-	src->free(src);
+	src->fn->free(src);
 
 	return 0;
 }
@@ -49,6 +49,13 @@ static void data_integer_print(const data_unset *d, int depth) {
 
 
 data_integer *data_integer_init(void) {
+	static const struct data_methods fn = {
+		data_integer_reset,
+		data_integer_copy,
+		data_integer_free,
+		data_integer_insert_dup,
+		data_integer_print,
+	};
 	data_integer *ds;
 
 	ds = calloc(1, sizeof(*ds));
@@ -57,12 +64,8 @@ data_integer *data_integer_init(void) {
 	ds->key = buffer_init();
 	ds->value = 0;
 
-	ds->copy = data_integer_copy;
-	ds->free = data_integer_free;
-	ds->reset = data_integer_reset;
-	ds->insert_dup = data_integer_insert_dup;
-	ds->print = data_integer_print;
 	ds->type = TYPE_INTEGER;
+	ds->fn = &fn;
 
 	return ds;
 }
