@@ -33,7 +33,7 @@
 
 #include "status_counter.h"
 
-static data_integer * gw_status_get_di(server *srv, gw_host *host, gw_proc *proc, const char *tag, size_t len) {
+static int * gw_status_get_counter(server *srv, gw_host *host, gw_proc *proc, const char *tag, size_t len) {
     buffer *b = srv->tmp_buf;
     buffer_copy_string_len(b, CONST_STR_LEN("gw.backend."));
     buffer_append_string_buffer(b, host->id);
@@ -46,42 +46,37 @@ static data_integer * gw_status_get_di(server *srv, gw_host *host, gw_proc *proc
 }
 
 static void gw_proc_tag_inc(server *srv, gw_host *host, gw_proc *proc, const char *tag, size_t len) {
-    data_integer *di = gw_status_get_di(srv, host, proc, tag, len);
-    ++di->value;
+    ++(*gw_status_get_counter(srv, host, proc, tag, len));
 }
 
 static void gw_proc_load_inc(server *srv, gw_host *host, gw_proc *proc) {
-    data_integer *di = gw_status_get_di(srv,host,proc,CONST_STR_LEN(".load"));
-    di->value = ++proc->load;
+    *gw_status_get_counter(srv,host,proc,CONST_STR_LEN(".load")) = ++proc->load;
 
     status_counter_inc(srv, CONST_STR_LEN("gw.active-requests"));
 }
 
 static void gw_proc_load_dec(server *srv, gw_host *host, gw_proc *proc) {
-    data_integer *di = gw_status_get_di(srv,host,proc,CONST_STR_LEN(".load"));
-    di->value = --proc->load;
+    *gw_status_get_counter(srv,host,proc,CONST_STR_LEN(".load")) = --proc->load;
 
     status_counter_dec(srv, CONST_STR_LEN("gw.active-requests"));
 }
 
 static void gw_host_assign(server *srv, gw_host *host) {
-    data_integer *di = gw_status_get_di(srv,host,NULL,CONST_STR_LEN(".load"));
-    di->value = ++host->load;
+    *gw_status_get_counter(srv,host,NULL,CONST_STR_LEN(".load")) = ++host->load;
 }
 
 static void gw_host_reset(server *srv, gw_host *host) {
-    data_integer *di = gw_status_get_di(srv,host,NULL,CONST_STR_LEN(".load"));
-    di->value = --host->load;
+    *gw_status_get_counter(srv,host,NULL,CONST_STR_LEN(".load")) = --host->load;
 }
 
 static int gw_status_init(server *srv, gw_host *host, gw_proc *proc) {
-    gw_status_get_di(srv, host, proc, CONST_STR_LEN(".disabled"))->value = 0;
-    gw_status_get_di(srv, host, proc, CONST_STR_LEN(".died"))->value = 0;
-    gw_status_get_di(srv, host, proc, CONST_STR_LEN(".overloaded"))->value = 0;
-    gw_status_get_di(srv, host, proc, CONST_STR_LEN(".connected"))->value = 0;
-    gw_status_get_di(srv, host, proc, CONST_STR_LEN(".load"))->value = 0;
+    *gw_status_get_counter(srv, host, proc, CONST_STR_LEN(".disabled")) = 0;
+    *gw_status_get_counter(srv, host, proc, CONST_STR_LEN(".died")) = 0;
+    *gw_status_get_counter(srv, host, proc, CONST_STR_LEN(".overloaded")) = 0;
+    *gw_status_get_counter(srv, host, proc, CONST_STR_LEN(".connected")) = 0;
+    *gw_status_get_counter(srv, host, proc, CONST_STR_LEN(".load")) = 0;
 
-    gw_status_get_di(srv, host, NULL, CONST_STR_LEN(".load"))->value = 0;
+    *gw_status_get_counter(srv, host, NULL, CONST_STR_LEN(".load")) = 0;
 
     return 0;
 }
@@ -2276,16 +2271,12 @@ handler_t gw_check_extension(server *srv, connection *con, gw_plugin_data *p, in
 
         /* check if extension-mapping matches */
         if (p->conf.ext_mapping) {
-            for (size_t k = 0; k < p->conf.ext_mapping->used; ++k) {
-                data_string *ds = (data_string *)p->conf.ext_mapping->data[k];
-                size_t ct_len = buffer_string_length(ds->key);
-                if (s_len < ct_len) continue;
-
+            data_string *ds =
+              (data_string *)array_match_key_suffix(p->conf.ext_mapping, fn);
+            if (NULL != ds) {
                 /* found a mapping */
-                if (0 == memcmp(fn->ptr+s_len-ct_len, ds->key->ptr, ct_len)) {
                     /* check if we know the extension */
-
-                    /* we can reuse k here */
+                    size_t k;
                     for (k = 0; k < exts->used; ++k) {
                         extension = exts->exts[k];
 
@@ -2298,8 +2289,6 @@ handler_t gw_check_extension(server *srv, connection *con, gw_plugin_data *p, in
                         /* found nothing */
                         extension = NULL;
                     }
-                    break;
-                }
             }
         }
 
