@@ -3,6 +3,7 @@
 #include "request.h"
 #include "base.h"
 #include "burl.h"
+#include "http_header.h"
 #include "http_kv.h"
 #include "log.h"
 #include "sock_addr.h"
@@ -12,6 +13,7 @@
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys-strings.h>
 
 static int request_check_hostname(buffer *host) {
 	enum { DOMAINLABEL, TOPLABEL } stage = TOPLABEL;
@@ -334,27 +336,18 @@ int http_request_host_policy (connection *con, buffer *b, const buffer *scheme)
 #define DUMP_HEADER
 #endif
 
-static int http_request_split_value(array *vals, buffer *b) {
-	size_t i, len;
+static int http_request_split_value(array *vals, const char *current, size_t len) {
 	int state = 0;
-
-	const char *current;
 	const char *token_start = NULL, *token_end = NULL;
 	/*
 	 * parse
 	 *
 	 * val1, val2, val3, val4
 	 *
-	 * into a array (more or less a explode() incl. striping of whitespaces
+	 * into a array (more or less a explode() incl. stripping of whitespaces
 	 */
 
-	if (buffer_string_is_empty(b)) return 0;
-
-	current = b->ptr;
-	len = buffer_string_length(b);
-	for (i =  0; i <= len; ++i, ++current) {
-		data_string *ds;
-
+	for (size_t i = 0; i <= len; ++i, ++current) {
 		switch (state) {
 		case 0: /* find start of a token */
 			switch (*current) {
@@ -379,13 +372,7 @@ static int http_request_split_value(array *vals, buffer *b) {
 				break;
 			case ',':
 			case '\0': /* end of string also marks the end of a token */
-				if (NULL == (ds = (data_string *)array_get_unused_element(vals, TYPE_STRING))) {
-					ds = data_string_init();
-				}
-
-				buffer_copy_string_len(ds->value, token_start, token_end-token_start+1);
-				array_insert_unique(vals, (data_unset *)ds);
-
+				array_insert_value(vals, token_start, token_end-token_start+1);
 				state = 0;
 				break;
 			default:
@@ -408,33 +395,180 @@ static int request_uri_is_valid_char(unsigned char c) {
 	return 1;
 }
 
-static int http_request_missing_CR_before_LF(server *srv, connection *con) {
+static void http_request_missing_CR_before_LF(server *srv, connection *con) {
 	if (srv->srvconf.log_request_header_on_error) {
 		log_error_write(srv, __FILE__, __LINE__, "s", "missing CR before LF in header -> 400");
 		log_error_write(srv, __FILE__, __LINE__, "Sb", "request-header:\n", con->request.request);
 	}
-
-	con->http_status = 400;
-	con->keep_alive = 0;
-	con->response.keep_alive = 0;
-	return 0;
 }
 
-int http_request_parse(server *srv, connection *con) {
-	char *uri = NULL, *proto = NULL, *method = NULL, con_length_set;
-	int is_key = 1, key_len = 0, is_ws_after_key = 0, in_folding;
-	char *value = NULL, *key = NULL;
-	char *reqline_host = NULL;
-	int reqline_hostlen = 0;
+enum keep_alive_set {
+	HTTP_CONNECTION_UNSET,
+	HTTP_CONNECTION_KEEPALIVE,
+	HTTP_CONNECTION_CLOSE,
+};
 
-	enum { HTTP_CONNECTION_UNSET, HTTP_CONNECTION_KEEPALIVE, HTTP_CONNECTION_CLOSE } keep_alive_set = HTTP_CONNECTION_UNSET;
+typedef struct {
+	enum keep_alive_set keep_alive_set;
+	char con_length_set;
+	char *reqline_host;
+	int reqline_hostlen;
+} parse_header_state;
 
+static void init_parse_header_state(parse_header_state* state) {
+	state->keep_alive_set = HTTP_CONNECTION_UNSET;
+	state->con_length_set = 0;
+	state->reqline_host = NULL;
+	state->reqline_hostlen = 0;
+}
+
+/* add header to list of headers
+ * certain headers are also parsed
+ * might drop a header if deemed unnecessary/broken
+ *
+ * returns 0 on error
+ */
+static int parse_single_header(server *srv, connection *con, parse_header_state *state, char *k, size_t klen, char *v, size_t vlen) {
+    const enum http_header_e id = http_header_hkey_get(k, klen);
+    buffer **saveb = NULL;
+
+    /* strip leading whitespace */
+    for (; vlen > 0 && (v[0] == ' ' || v[0] == '\t'); ++v, --vlen) ;
+
+    /* strip trailing whitespace */
+    while (vlen > 0 && (v[vlen - 1] == ' ' || v[vlen - 1] == '\t')) --vlen;
+
+    /* empty header-fields are not allowed by HTTP-RFC, we just ignore them */
+    if (0 == vlen) return 1; /* ignore header */
+
+    /*
+     * Note: k might not be '\0'-terminated
+     */
+
+    switch (id) {
+      /*case HTTP_HEADER_OTHER:*/
+      default:
+        break;
+      case HTTP_HEADER_HOST:
+        if (!(con->request.htags & HTTP_HEADER_HOST)) {
+            saveb = &con->request.http_host;
+        }
+        else if (state->reqline_host) {
+            /* ignore all Host: headers as we got Host in request line */
+            return 1; /* ignore header */
+        }
+        else {
+            if (srv->srvconf.log_request_header_on_error) {
+                log_error_write(srv, __FILE__, __LINE__, "s",
+                                "duplicate Host-header -> 400");
+                log_error_write(srv, __FILE__, __LINE__, "Sb",
+                                "request-header:\n", con->request.request);
+            }
+            return 0; /* invalid header */
+        }
+        break;
+      case HTTP_HEADER_CONNECTION:
+        {
+            array * const vals = srv->split_vals;
+            array_reset(vals);
+            http_request_split_value(vals, v, vlen); /* split on , */
+            for (size_t vi = 0; vi < vals->used; ++vi) {
+                data_string *dsv = (data_string *)vals->data[vi];
+                if (0 == buffer_caseless_compare(CONST_BUF_LEN(dsv->value),
+                                                 CONST_STR_LEN("keep-alive"))) {
+                    state->keep_alive_set = HTTP_CONNECTION_KEEPALIVE;
+                    break;
+                }
+                else if (0 == buffer_caseless_compare(CONST_BUF_LEN(dsv->value),
+                                                      CONST_STR_LEN("close"))) {
+                    state->keep_alive_set = HTTP_CONNECTION_CLOSE;
+                    break;
+                }
+            }
+        }
+        break;
+      case HTTP_HEADER_CONTENT_TYPE:
+        if (con->request.htags & HTTP_HEADER_CONTENT_TYPE) {
+            if (srv->srvconf.log_request_header_on_error) {
+                log_error_write(srv, __FILE__, __LINE__, "s",
+                                "duplicate Content-Type-header -> 400");
+                log_error_write(srv, __FILE__, __LINE__, "Sb",
+                                "request-header:\n", con->request.request);
+            }
+            return 0; /* invalid header */
+        }
+        break;
+      case HTTP_HEADER_IF_NONE_MATCH:
+        /* if dup, only the first one will survive */
+        if (con->request.htags & HTTP_HEADER_IF_NONE_MATCH) {
+            return 1; /* ignore header */
+        }
+        break;
+      case HTTP_HEADER_CONTENT_LENGTH:
+        if (!(con->request.htags & HTTP_HEADER_CONTENT_LENGTH)) {
+            char *err;
+            off_t r = strtoll(v, &err, 10);
+
+            if (*err == '\0' && r >= 0) {
+                con->request.content_length = r;
+            }
+            else {
+                log_error_write(srv, __FILE__, __LINE__, "sss",
+                                "content-length broken:", v, "-> 400");
+                return 0; /* invalid header */
+            }
+        }
+        else {
+            if (srv->srvconf.log_request_header_on_error) {
+                log_error_write(srv, __FILE__, __LINE__, "s",
+                                "duplicate Content-Length-header -> 400");
+                log_error_write(srv, __FILE__, __LINE__, "Sb",
+                                "request-header:\n", con->request.request);
+            }
+            return 0; /* invalid header */
+        }
+        break;
+      case HTTP_HEADER_IF_MODIFIED_SINCE:
+        if (con->request.htags & HTTP_HEADER_IF_MODIFIED_SINCE) {
+            /* Proxies sometimes send dup headers
+             * if they are the same we ignore the second
+             * if not, we raise an error */
+            buffer *vb =
+              http_header_request_get(con, HTTP_HEADER_IF_MODIFIED_SINCE,
+                                      CONST_STR_LEN("If-Modified-Since"));
+            if (vb && buffer_is_equal_caseless_string(vb, v, vlen)) {
+                /* ignore it if they are the same */
+                return 1; /* ignore header */
+            }
+            else {
+                if (srv->srvconf.log_request_header_on_error) {
+                    log_error_write(srv, __FILE__, __LINE__, "s",
+                                    "duplicate If-Modified-Since header -> 400");
+                    log_error_write(srv, __FILE__, __LINE__, "Sb",
+                                    "request-header:\n", con->request.request);
+                }
+                return 0; /* invalid header */
+            }
+        }
+        break;
+    }
+
+    con->request.htags |= id;
+    http_header_request_append(con, id, k, klen, v, vlen);
+
+    if (saveb) {
+        *saveb = http_header_request_get(con, id, k, klen);
+    }
+
+    return 1;
+}
+
+static size_t http_request_parse_reqline(server *srv, connection *con, parse_header_state *state) {
+	char *uri = NULL, *proto = NULL, *method = NULL;
 	int line = 0;
 
 	int request_line_stage = 0;
 	size_t i, first, ilen;
-
-	int done = 0;
 	const unsigned int http_header_strict = (con->conf.http_parseopts & HTTP_PARSEOPT_HEADER_STRICT);
 
 	/*
@@ -457,8 +591,6 @@ int http_request_parse(server *srv, connection *con) {
 
 	      #ifdef __COVERITY__
 		if (buffer_string_length(con->request.request) < 2) {
-			con->keep_alive = 0;
-			con->http_status = 400;
 			return 0;
 		}
 	      #endif
@@ -467,11 +599,12 @@ int http_request_parse(server *srv, connection *con) {
 	} else if (con->request_count > 0 &&
 	    con->request.request->ptr[1] == '\n') {
 		/* we are in keep-alive and might get \n after a previous POST request.*/
-		if (http_header_strict) return http_request_missing_CR_before_LF(srv, con);
+		if (http_header_strict) {
+			http_request_missing_CR_before_LF(srv, con);
+			return 0;
+		}
 	      #ifdef __COVERITY__
 		if (buffer_string_length(con->request.request) < 1) {
-			con->keep_alive = 0;
-			con->http_status = 400;
 			return 0;
 		}
 	      #endif
@@ -482,9 +615,6 @@ int http_request_parse(server *srv, connection *con) {
 		buffer_copy_buffer(con->parse_request, con->request.request);
 	}
 
-	keep_alive_set = 0;
-	con_length_set = 0;
-
 	/* parse the first line of the request
 	 *
 	 * should be:
@@ -510,15 +640,12 @@ int http_request_parse(server *srv, connection *con) {
 					con->parse_request->ptr[i] = '\0';
 					++i;
 				} else if (http_header_strict) { /* '\n' */
-					return http_request_missing_CR_before_LF(srv, con);
+					http_request_missing_CR_before_LF(srv, con);
+					return 0;
 				}
 				con->parse_request->ptr[i] = '\0';
 
 				if (request_line_stage != 2) {
-					con->http_status = 400;
-					con->response.keep_alive = 0;
-					con->keep_alive = 0;
-
 					if (srv->srvconf.log_request_header_on_error) {
 						log_error_write(srv, __FILE__, __LINE__, "s", "incomplete request line -> 400");
 						log_error_write(srv, __FILE__, __LINE__, "Sb",
@@ -536,8 +663,6 @@ int http_request_parse(server *srv, connection *con) {
 				/* we got the first one :) */
 				if (HTTP_METHOD_UNSET == (r = get_http_method_key(method))) {
 					con->http_status = 501;
-					con->response.keep_alive = 0;
-					con->keep_alive = 0;
 
 					if (srv->srvconf.log_request_header_on_error) {
 						log_error_write(srv, __FILE__, __LINE__, "s", "unknown http-method -> 501");
@@ -582,9 +707,6 @@ int http_request_parse(server *srv, connection *con) {
 					}
 
 					if (invalid_version) {
-						con->http_status = 400;
-						con->keep_alive = 0;
-
 						if (srv->srvconf.log_request_header_on_error) {
 							log_error_write(srv, __FILE__, __LINE__, "s", "unknown protocol -> 400");
 							log_error_write(srv, __FILE__, __LINE__, "Sb",
@@ -610,9 +732,6 @@ int http_request_parse(server *srv, connection *con) {
 						return 0;
 					}
 				} else {
-					con->http_status = 400;
-					con->keep_alive = 0;
-
 					if (srv->srvconf.log_request_header_on_error) {
 						log_error_write(srv, __FILE__, __LINE__, "s", "unknown protocol -> 400");
 						log_error_write(srv, __FILE__, __LINE__, "Sb",
@@ -625,16 +744,16 @@ int http_request_parse(server *srv, connection *con) {
 				if (*uri == '/') {
 					/* (common case) */
 					buffer_copy_string_len(con->request.uri, uri, proto - uri - 1);
-				} else if (0 == strncasecmp(uri, "http://", 7) &&
+				} else if (0 == buffer_caseless_compare(uri, 7, "http://", 7) &&
 				    NULL != (nuri = strchr(uri + 7, '/'))) {
-					reqline_host = uri + 7;
-					reqline_hostlen = nuri - reqline_host;
+					state->reqline_host = uri + 7;
+					state->reqline_hostlen = nuri - state->reqline_host;
 
 					buffer_copy_string_len(con->request.uri, nuri, proto - nuri - 1);
-				} else if (0 == strncasecmp(uri, "https://", 8) &&
+				} else if (0 == buffer_caseless_compare(uri, 8, "https://", 8) &&
 				    NULL != (nuri = strchr(uri + 8, '/'))) {
-					reqline_host = uri + 8;
-					reqline_hostlen = nuri - reqline_host;
+					state->reqline_host = uri + 8;
+					state->reqline_hostlen = nuri - state->reqline_host;
 
 					buffer_copy_string_len(con->request.uri, nuri, proto - nuri - 1);
 				} else if (!http_header_strict
@@ -643,8 +762,6 @@ int http_request_parse(server *srv, connection *con) {
 					/* everything looks good so far */
 					buffer_copy_string_len(con->request.uri, uri, proto - uri - 1);
 				} else {
-					con->http_status = 400;
-					con->keep_alive = 0;
 					log_error_write(srv, __FILE__, __LINE__, "ss", "request-URI parse error -> 400 for:", uri);
 					return 0;
 				}
@@ -660,33 +777,30 @@ int http_request_parse(server *srv, connection *con) {
 					j = (NULL == z) ? jlen : (size_t)(z - con->request.uri->ptr);
 				}
 				if (j < jlen) {
-						con->http_status = 400;
-						con->keep_alive = 0;
+					if (srv->srvconf.log_request_header_on_error) {
+						unsigned char buf[2];
+						buf[0] = con->request.uri->ptr[j];
+						buf[1] = '\0';
 
-						if (srv->srvconf.log_request_header_on_error) {
-							unsigned char buf[2];
-							buf[0] = con->request.uri->ptr[j];
-							buf[1] = '\0';
-
-							if (con->request.uri->ptr[j] > 32 &&
-							    con->request.uri->ptr[j] != 127) {
-								/* the character is printable -> print it */
-								log_error_write(srv, __FILE__, __LINE__, "ss",
-										"invalid character in URI -> 400",
-										buf);
-							} else {
-								/* a control-character, print ascii-code */
-								log_error_write(srv, __FILE__, __LINE__, "sd",
-										"invalid character in URI -> 400",
-										con->request.uri->ptr[j]);
-							}
-
-							log_error_write(srv, __FILE__, __LINE__, "Sb",
-									"request-header:\n",
-									con->request.request);
+						if (con->request.uri->ptr[j] > 32 &&
+							con->request.uri->ptr[j] != 127) {
+							/* the character is printable -> print it */
+							log_error_write(srv, __FILE__, __LINE__, "ss",
+									"invalid character in URI -> 400",
+									buf);
+						} else {
+							/* a control-character, print ascii-code */
+							log_error_write(srv, __FILE__, __LINE__, "sd",
+									"invalid character in URI -> 400",
+									con->request.uri->ptr[j]);
 						}
 
-						return 0;
+						log_error_write(srv, __FILE__, __LINE__, "Sb",
+								"request-header:\n",
+								con->request.request);
+					}
+
+					return 0;
 				}
 
 				buffer_copy_buffer(con->request.orig_uri, con->request.uri);
@@ -711,10 +825,6 @@ int http_request_parse(server *srv, connection *con) {
 				break;
 			default:
 				/* ERROR, one space to much */
-				con->http_status = 400;
-				con->response.keep_alive = 0;
-				con->keep_alive = 0;
-
 				if (srv->srvconf.log_request_header_on_error) {
 					log_error_write(srv, __FILE__, __LINE__, "s", "overlong request line -> 400");
 					log_error_write(srv, __FILE__, __LINE__, "Sb",
@@ -729,13 +839,7 @@ int http_request_parse(server *srv, connection *con) {
 		}
 	}
 
-	in_folding = 0;
-
 	if (buffer_string_is_empty(con->request.uri)) {
-		con->http_status = 400;
-		con->response.keep_alive = 0;
-		con->keep_alive = 0;
-
 		if (srv->srvconf.log_request_header_on_error) {
 			log_error_write(srv, __FILE__, __LINE__, "s", "no uri specified -> 400");
 			log_error_write(srv, __FILE__, __LINE__, "Sb",
@@ -745,43 +849,65 @@ int http_request_parse(server *srv, connection *con) {
 		return 0;
 	}
 
-	if (reqline_host) {
+	if (state->reqline_host) {
 		/* Insert as host header */
-		data_string *ds;
-
-		if (NULL == (ds = (data_string *)array_get_unused_element(con->request.headers, TYPE_STRING))) {
-			ds = data_string_init();
-		}
-
-		buffer_copy_string_len(ds->key, CONST_STR_LEN("Host"));
-		buffer_copy_string_len(ds->value, reqline_host, reqline_hostlen);
-		array_insert_unique(con->request.headers, (data_unset *)ds);
-		con->request.http_host = ds->value;
+		http_header_request_set(con, HTTP_HEADER_HOST, CONST_STR_LEN("Host"), state->reqline_host, state->reqline_hostlen);
+		con->request.http_host = http_header_request_get(con, HTTP_HEADER_HOST, CONST_STR_LEN("Host"));
 	}
 
-	for (; i <= ilen && !done; i++) {
+	return i;
+}
+
+int http_request_parse(server *srv, connection *con) {
+	char *value = NULL;
+	size_t i, first, ilen;
+	const unsigned int http_header_strict = (con->conf.http_parseopts & HTTP_PARSEOPT_HEADER_STRICT);
+
+	parse_header_state state;
+	init_parse_header_state(&state);
+
+	i = first = http_request_parse_reqline(srv, con, &state);
+	if (0 == i) goto failure;
+
+	if (con->parse_request->ptr[i] == ' ' || con->parse_request->ptr[i] == '\t') {
+		if (srv->srvconf.log_request_header_on_error) {
+			log_error_write(srv, __FILE__, __LINE__, "s", "WS at the start of first line -> 400");
+			log_error_write(srv, __FILE__, __LINE__, "Sb", "request-header:\n", con->request.request);
+		}
+		goto failure;
+	}
+
+	ilen = buffer_string_length(con->parse_request);
+	for (int is_key = 1, key_len = 0, done = 0; i <= ilen && !done; ++i) {
 		char *cur = con->parse_request->ptr + i;
 
 		if (is_key) {
-			size_t j;
-			int got_colon = 0;
-
 			/**
 			 * 1*<any CHAR except CTLs or separators>
 			 * CTLs == 0-31 + 127, CHAR = 7-bit ascii (0..127)
 			 *
 			 */
 			switch(*cur) {
+			case ' ':
+			case '\t':
+				/* skip every thing up to the : */
+				do { ++cur; } while (*cur == ' ' || *cur == '\t');
+				if (*cur != ':') {
+						if (srv->srvconf.log_request_header_on_error) {
+							log_error_write(srv, __FILE__, __LINE__, "s", "WS character in key -> 400");
+							log_error_write(srv, __FILE__, __LINE__, "Sb",
+								"request-header:\n",
+								con->request.request);
+						}
+
+						goto failure;
+				}
+				/* fall through */
 			case ':':
 				is_key = 0;
-
+				key_len = i - first;
 				value = cur + 1;
-
-				if (is_ws_after_key == 0) {
-					key_len = i - first;
-				}
-				is_ws_after_key = 0;
-
+				i = cur - con->parse_request->ptr;
 				break;
 			case '(':
 			case ')':
@@ -799,10 +925,6 @@ int http_request_parse(server *srv, connection *con) {
 			case '=':
 			case '{':
 			case '}':
-				con->http_status = 400;
-				con->keep_alive = 0;
-				con->response.keep_alive = 0;
-
 				if (srv->srvconf.log_request_header_on_error) {
 					log_error_write(srv, __FILE__, __LINE__, "sbsds",
 						"invalid character in key", con->request.request, cur, *cur, "-> 400");
@@ -811,54 +933,7 @@ int http_request_parse(server *srv, connection *con) {
 						"request-header:\n",
 						con->request.request);
 				}
-				return 0;
-			case ' ':
-			case '\t':
-				if (i == first) {
-					is_key = 0;
-					in_folding = 1;
-					value = cur;
-
-					break;
-				}
-
-
-				key_len = i - first;
-
-				/* skip every thing up to the : */
-				for (j = 1; !got_colon; j++) {
-					switch(con->parse_request->ptr[j + i]) {
-					case ' ':
-					case '\t':
-						/* skip WS */
-						continue;
-					case ':':
-						/* ok, done; handle the colon the usual way */
-
-						i += j - 1;
-						got_colon = 1;
-						is_ws_after_key = 1; /* we already know the key length */
-
-						break;
-					default:
-						/* error */
-
-						if (srv->srvconf.log_request_header_on_error) {
-							log_error_write(srv, __FILE__, __LINE__, "s", "WS character in key -> 400");
-							log_error_write(srv, __FILE__, __LINE__, "Sb",
-								"request-header:\n",
-								con->request.request);
-						}
-
-						con->http_status = 400;
-						con->response.keep_alive = 0;
-						con->keep_alive = 0;
-
-						return 0;
-					}
-				}
-
-				break;
+				goto failure;
 			case '\r':
 				if (con->parse_request->ptr[i+1] == '\n' && i == first) {
 					/* End of Header */
@@ -876,15 +951,13 @@ int http_request_parse(server *srv, connection *con) {
 							con->request.request);
 					}
 
-					con->http_status = 400;
-					con->keep_alive = 0;
-					con->response.keep_alive = 0;
-					return 0;
+					goto failure;
 				}
 				break;
 			case '\n':
 				if (http_header_strict) {
-					return http_request_missing_CR_before_LF(srv, con);
+					http_request_missing_CR_before_LF(srv, con);
+					goto failure;
 				} else if (i == first) {
 					con->parse_request->ptr[i] = '\0';
 					done = 1;
@@ -893,10 +966,6 @@ int http_request_parse(server *srv, connection *con) {
 				/* fall through */
 			default:
 				if (http_header_strict ? (*cur < 32 || ((unsigned char)*cur) >= 127) : *cur == '\0') {
-					con->http_status = 400;
-					con->keep_alive = 0;
-					con->response.keep_alive = 0;
-
 					if (srv->srvconf.log_request_header_on_error) {
 						log_error_write(srv, __FILE__, __LINE__, "sbsds",
 							"invalid character in key", con->request.request, cur, *cur, "-> 400");
@@ -906,7 +975,7 @@ int http_request_parse(server *srv, connection *con) {
 							con->request.request);
 					}
 
-					return 0;
+					goto failure;
 				}
 				/* ok */
 				break;
@@ -914,244 +983,49 @@ int http_request_parse(server *srv, connection *con) {
 		} else {
 			switch(*cur) {
 			case '\r':
-			case '\n':
-				if (*cur == '\n' || con->parse_request->ptr[i+1] == '\n') {
-					data_string *ds = NULL;
-					if (*cur == '\n') {
-						if (http_header_strict) return http_request_missing_CR_before_LF(srv, con);
-					} else { /* (con->parse_request->ptr[i+1] == '\n') */
-						con->parse_request->ptr[i] = '\0';
-						++i;
-					}
-
-					/* End of Headerline */
-					con->parse_request->ptr[i] = '\0';
-
-					if (in_folding) {
-						/**
-						 * we use a evil hack to handle the line-folding
-						 * 
-						 * As array_insert_unique() deletes 'ds' in the case of a duplicate
-						 * ds points somewhere and we get a evil crash. As a solution we keep the old
-						 * "key" and get the current value from the hash and append us
-						 *
-						 * */
-
-						if (!key || !key_len) {
-							/* 400 */
-
-							if (srv->srvconf.log_request_header_on_error) {
-								log_error_write(srv, __FILE__, __LINE__, "s", "WS at the start of first line -> 400");
-
-								log_error_write(srv, __FILE__, __LINE__, "Sb",
-									"request-header:\n",
-									con->request.request);
-							}
-
-
-							con->http_status = 400;
-							con->keep_alive = 0;
-							con->response.keep_alive = 0;
-							return 0;
-						}
-
-						if (NULL != (ds = (data_string *)array_get_element_klen(con->request.headers, key, key_len))) {
-							buffer_append_string(ds->value, value);
-						}
-					} else {
-						int s_len;
-						key = con->parse_request->ptr + first;
-
-						s_len = cur - value;
-
-						/* strip trailing white-spaces */
-						for (; s_len > 0 && 
-								(value[s_len - 1] == ' ' || 
-								 value[s_len - 1] == '\t'); s_len--);
-
-						value[s_len] = '\0';
-
-						if (s_len > 0) {
-							int cmp = 0;
-							if (NULL == (ds = (data_string *)array_get_unused_element(con->request.headers, TYPE_STRING))) {
-								ds = data_string_init();
-							}
-							buffer_copy_string_len(ds->key, key, key_len);
-							buffer_copy_string_len(ds->value, value, s_len);
-
-							/* retreive values
-							 *
-							 *
-							 * the list of options is sorted to simplify the search
-							 */
-
-							if (0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("Connection")))) {
-								array *vals;
-								size_t vi;
-
-								/* split on , */
-
-								vals = srv->split_vals;
-
-								array_reset(vals);
-
-								http_request_split_value(vals, ds->value);
-
-								for (vi = 0; vi < vals->used; vi++) {
-									data_string *dsv = (data_string *)vals->data[vi];
-
-									if (0 == buffer_caseless_compare(CONST_BUF_LEN(dsv->value), CONST_STR_LEN("keep-alive"))) {
-										keep_alive_set = HTTP_CONNECTION_KEEPALIVE;
-
-										break;
-									} else if (0 == buffer_caseless_compare(CONST_BUF_LEN(dsv->value), CONST_STR_LEN("close"))) {
-										keep_alive_set = HTTP_CONNECTION_CLOSE;
-
-										break;
-									}
-								}
-
-							} else if (cmp > 0 && 0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("Content-Length")))) {
-								char *err;
-								off_t r;
-
-								if (con_length_set) {
-									con->http_status = 400;
-									con->keep_alive = 0;
-
-									if (srv->srvconf.log_request_header_on_error) {
-										log_error_write(srv, __FILE__, __LINE__, "s",
-												"duplicate Content-Length-header -> 400");
-										log_error_write(srv, __FILE__, __LINE__, "Sb",
-												"request-header:\n",
-												con->request.request);
-									}
-									array_insert_unique(con->request.headers, (data_unset *)ds);
-									return 0;
-								}
-
-								r = strtoll(ds->value->ptr, &err, 10);
-
-								if (*err == '\0' && r >= 0) {
-									con_length_set = 1;
-									con->request.content_length = r;
-								} else {
-									log_error_write(srv, __FILE__, __LINE__, "sbs",
-											"content-length broken:", ds->value, "-> 400");
-
-									con->http_status = 400;
-									con->keep_alive = 0;
-
-									array_insert_unique(con->request.headers, (data_unset *)ds);
-									return 0;
-								}
-							} else if (cmp > 0 && 0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("Content-Type")))) {
-								/* if dup, only the first one will survive */
-								if (!con->request.http_content_type) {
-									con->request.http_content_type = ds->value->ptr;
-								} else {
-									con->http_status = 400;
-									con->keep_alive = 0;
-
-									if (srv->srvconf.log_request_header_on_error) {
-										log_error_write(srv, __FILE__, __LINE__, "s",
-												"duplicate Content-Type-header -> 400");
-										log_error_write(srv, __FILE__, __LINE__, "Sb",
-												"request-header:\n",
-												con->request.request);
-									}
-									array_insert_unique(con->request.headers, (data_unset *)ds);
-									return 0;
-								}
-							} else if (cmp > 0 && 0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("Host")))) {
-								if (reqline_host) {
-									/* ignore all host: headers as we got the host in the request line */
-									ds->free((data_unset*) ds);
-									ds = NULL;
-								} else if (!con->request.http_host) {
-									con->request.http_host = ds->value;
-								} else {
-									con->http_status = 400;
-									con->keep_alive = 0;
-
-									if (srv->srvconf.log_request_header_on_error) {
-										log_error_write(srv, __FILE__, __LINE__, "s",
-												"duplicate Host-header -> 400");
-										log_error_write(srv, __FILE__, __LINE__, "Sb",
-												"request-header:\n",
-												con->request.request);
-									}
-									array_insert_unique(con->request.headers, (data_unset *)ds);
-									return 0;
-								}
-							} else if (cmp > 0 && 0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("If-Modified-Since")))) {
-								/* Proxies sometimes send dup headers
-								 * if they are the same we ignore the second
-								 * if not, we raise an error */
-								if (!con->request.http_if_modified_since) {
-									con->request.http_if_modified_since = ds->value->ptr;
-								} else if (0 == strcasecmp(con->request.http_if_modified_since,
-											ds->value->ptr)) {
-									/* ignore it if they are the same */
-
-									ds->free((data_unset *)ds);
-									ds = NULL;
-								} else {
-									con->http_status = 400;
-									con->keep_alive = 0;
-
-									if (srv->srvconf.log_request_header_on_error) {
-										log_error_write(srv, __FILE__, __LINE__, "s",
-												"duplicate If-Modified-Since header -> 400");
-										log_error_write(srv, __FILE__, __LINE__, "Sb",
-												"request-header:\n",
-												con->request.request);
-									}
-									array_insert_unique(con->request.headers, (data_unset *)ds);
-									return 0;
-								}
-							} else if (cmp > 0 && 0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("If-None-Match")))) {
-								/* if dup, only the first one will survive */
-								if (!con->request.http_if_none_match) {
-									con->request.http_if_none_match = ds->value->ptr;
-								} else {
-									ds->free((data_unset*) ds);
-									ds = NULL;
-								}
-							}
-
-							if (ds) array_insert_unique(con->request.headers, (data_unset *)ds);
-						} else {
-							/* empty header-fields are not allowed by HTTP-RFC, we just ignore them */
-						}
-					}
-
-					first = i+1;
-					is_key = 1;
-					value = NULL;
-#if 0
-					/**
-					 * for Bug 1230 keep the key_len a live
-					 */
-					key_len = 0; 
-#endif
-					in_folding = 0;
-				} else {
+				if (cur[1] != '\n') {
 					if (srv->srvconf.log_request_header_on_error) {
 						log_error_write(srv, __FILE__, __LINE__, "sbs",
 								"CR without LF", con->request.request, "-> 400");
 					}
 
-					con->http_status = 400;
-					con->keep_alive = 0;
-					con->response.keep_alive = 0;
-					return 0;
+					goto failure;
 				}
+				if (cur[2] == ' ' || cur[2] == '\t') { /* header line folding */
+					cur[0] = ' ';
+					cur[1] = ' ';
+					i += 2;
+					continue;
+				}
+				++i;
+				/* fall through */
+			case '\n':
+					if (*cur == '\n') {
+						if (http_header_strict) {
+							http_request_missing_CR_before_LF(srv, con);
+							goto failure;
+						}
+						if (cur[1] == ' ' || cur[1] == '\t') { /* header line folding */
+							cur[0] = ' ';
+							i += 1;
+							continue;
+						}
+					}
+
+					/* End of Headerline */
+					*cur = '\0'; /*(for if value is further parsed and '\0' is expected at end of string)*/
+
+					if (!parse_single_header(srv, con, &state, con->parse_request->ptr + first, key_len, value, cur - value)) {
+						/* parse_single_header should already have logged it */
+						goto failure;
+					}
+
+					first = i+1;
+					is_key = 1;
+					value = NULL;
 				break;
 			case ' ':
 			case '\t':
-				/* strip leading WS */
-				if (value == cur) value = cur+1;
 				break;
 			default:
 				if (http_header_strict ? (*cur >= 0 && *cur < 32) : *cur == '\0') {
@@ -1160,10 +1034,7 @@ int http_request_parse(server *srv, connection *con) {
 								"invalid char in header", (int)*cur, "-> 400");
 					}
 
-					con->http_status = 400;
-					con->keep_alive = 0;
-
-					return 0;
+					goto failure;
 				}
 				break;
 			}
@@ -1175,7 +1046,7 @@ int http_request_parse(server *srv, connection *con) {
 	/* do some post-processing */
 
 	if (con->request.http_version == HTTP_VERSION_1_1) {
-		if (keep_alive_set != HTTP_CONNECTION_CLOSE) {
+		if (state.keep_alive_set != HTTP_CONNECTION_CLOSE) {
 			/* no Connection-Header sent */
 
 			/* HTTP/1.1 -> keep-alive default TRUE */
@@ -1187,9 +1058,6 @@ int http_request_parse(server *srv, connection *con) {
 		/* RFC 2616, 14.23 */
 		if (con->request.http_host == NULL ||
 		    buffer_string_is_empty(con->request.http_host)) {
-			con->http_status = 400;
-			con->response.keep_alive = 0;
-			con->keep_alive = 0;
 
 			if (srv->srvconf.log_request_header_on_error) {
 				log_error_write(srv, __FILE__, __LINE__, "s", "HTTP/1.1 but Host missing -> 400");
@@ -1197,10 +1065,10 @@ int http_request_parse(server *srv, connection *con) {
 						"request-header:\n",
 						con->request.request);
 			}
-			return 0;
+			goto failure;
 		}
 	} else {
-		if (keep_alive_set == HTTP_CONNECTION_KEEPALIVE) {
+		if (state.keep_alive_set == HTTP_CONNECTION_KEEPALIVE) {
 			/* no Connection-Header sent */
 
 			/* HTTP/1.0 -> keep-alive default FALSE  */
@@ -1222,70 +1090,64 @@ int http_request_parse(server *srv, connection *con) {
 					con->request.request);
 		}
 
-		con->http_status = 400;
-		con->response.keep_alive = 0;
-		con->keep_alive = 0;
-
-		return 0;
+		goto failure;
 	}
 
-	{
-		data_string *ds = (data_string *)array_get_element(con->request.headers, "Transfer-Encoding");
-		if (NULL != ds) {
+        if (con->request.htags & HTTP_HEADER_TRANSFER_ENCODING) {
+		buffer *vb = http_header_request_get(con, HTTP_HEADER_TRANSFER_ENCODING, CONST_STR_LEN("Transfer-Encoding"));
+		if (NULL != vb) {
 			if (con->request.http_version == HTTP_VERSION_1_0) {
 				log_error_write(srv, __FILE__, __LINE__, "s",
 						"HTTP/1.0 with Transfer-Encoding (bad HTTP/1.0 proxy?) -> 400");
-				con->keep_alive = 0;
-				con->http_status = 400; /* Bad Request */
-				return 0;
+				goto failure;
 			}
 
-			if (0 != strcasecmp(ds->value->ptr, "chunked")) {
+			if (0 != strcasecmp(vb->ptr, "chunked")) {
 				/* Transfer-Encoding might contain additional encodings,
 				 * which are not currently supported by lighttpd */
-				con->keep_alive = 0;
 				con->http_status = 501; /* Not Implemented */
-				return 0;
+				goto failure;
 			}
 
 			/* reset value for Transfer-Encoding, a hop-by-hop header,
 			 * which must not be blindly forwarded to backends */
-			buffer_reset(ds->value); /* headers with empty values are ignored */
-
-			con_length_set = 1;
-			con->request.content_length = -1;
+			buffer_reset(vb); /* headers with empty values are ignored */
 
 			/*(note: ignore whether or not Content-Length was provided)*/
-			ds = (data_string *)array_get_element(con->request.headers, "Content-Length");
-			if (NULL != ds) buffer_reset(ds->value); /* headers with empty values are ignored */
+		        if (con->request.htags & HTTP_HEADER_CONTENT_LENGTH) {
+				vb = http_header_request_get(con, HTTP_HEADER_CONTENT_LENGTH, CONST_STR_LEN("Content-Length"));
+				if (NULL != vb) buffer_reset(vb); /* headers with empty values are ignored */
+			}
+
+			state.con_length_set = 1;
+			con->request.content_length = -1;
 		}
 	}
+        else if (con->request.htags & HTTP_HEADER_CONTENT_LENGTH) {
+		state.con_length_set = 1;
+	}
 
 	switch(con->request.http_method) {
 	case HTTP_METHOD_GET:
 	case HTTP_METHOD_HEAD:
 		/* content-length is forbidden for those */
-		if (con_length_set && con->request.content_length != 0) {
+		if (state.con_length_set && con->request.content_length != 0) {
 			/* content-length is missing */
 			log_error_write(srv, __FILE__, __LINE__, "s",
 					"GET/HEAD with content-length -> 400");
 
-			con->keep_alive = 0;
-			con->http_status = 400;
-			return 0;
+			goto failure;
 		}
 		break;
 	case HTTP_METHOD_POST:
 		/* content-length is required for them */
-		if (!con_length_set) {
+		if (!state.con_length_set) {
 			/* content-length is missing */
 			log_error_write(srv, __FILE__, __LINE__, "s",
 					"POST-request, but content-length missing -> 411");
 
-			con->keep_alive = 0;
 			con->http_status = 411;
-			return 0;
-
+			goto failure;
 		}
 		break;
 	default:
@@ -1294,12 +1156,19 @@ int http_request_parse(server *srv, connection *con) {
 
 
 	/* check if we have read post data */
-	if (con_length_set) {
+	if (state.con_length_set) {
 		/* we have content */
 		if (con->request.content_length != 0) {
 			return 1;
 		}
 	}
 
+	return 0;
+
+failure:
+	con->keep_alive = 0;
+	con->response.keep_alive = 0;
+	if (!con->http_status) con->http_status = 400;
+
 	return 0;
 }
