@@ -11,10 +11,6 @@
 int http_response_parse(server *srv, connection *con);
 int http_response_write_header(server *srv, connection *con);
 
-int response_header_insert(server *srv, connection *con, const char *key, size_t keylen, const char *value, size_t vallen);
-int response_header_overwrite(server *srv, connection *con, const char *key, size_t keylen, const char *value, size_t vallen);
-int response_header_append(server *srv, connection *con, const char *key, size_t keylen, const char *value, size_t vallen);
-
 typedef struct http_cgi_opts_t {
   int authorizer;
   int break_scriptfilename_for_php;
@@ -50,6 +46,7 @@ handler_t http_response_read(server *srv, connection *con, http_response_opts *o
 handler_t http_response_prepare(server *srv, connection *con);
 int http_response_redirect_to_directory(server *srv, connection *con);
 int http_response_handle_cachable(server *srv, connection *con, buffer * mtime);
+void http_response_body_clear(connection *con, int preserve_length);
 void http_response_send_file (server *srv, connection *con, buffer *path);
 void http_response_backend_done (server *srv, connection *con);
 void http_response_backend_error (server *srv, connection *con);
