@@ -71,7 +71,8 @@
 # include <sys/prctl.h>
 #endif
 
-#if defined HAVE_LIBSSL && defined HAVE_OPENSSL_SSL_H
+#include "sys-crypto.h"
+#ifdef USE_OPENSSL_CRYPTO
 #define USE_SSL
 #define TEXT_SSL " (ssl)"
 #else
@@ -588,6 +589,11 @@ static void show_features (void) {
 #else
       "\t- LDAP support\n"
 #endif
+#ifdef HAVE_PAM
+      "\t+ PAM support\n"
+#else
+      "\t- PAM support\n"
+#endif
 #ifdef USE_MEMCACHED
       "\t+ memcached support\n"
 #else
@@ -1050,7 +1056,7 @@ static int server_main (server * const srv, int argc, char **argv) {
 	if (print_config) {
 		data_unset *dc = srv->config_context->data[0];
 		if (dc) {
-			dc->print(dc, 0);
+			dc->fn->print(dc, 0);
 			fprintf(stdout, "\n");
 		} else {
 			/* shouldn't happend */
